export interface Events {
    [name: string]: Actions;
}
export declare type Action = Invoke | Export | Set | Navigate | Back | Notify | Execute | InvokeMethod | Popup | ClosePopup;
export declare type Actions = Array<Action>;
export interface ClosePopup {
    type: 'close-popup';
    result: string;
    condition?: string;
}
export interface InvokeMethodSchema {
    type: string;
    async: boolean;
    dotnetType: string;
}
export interface InvokeMethod {
    type: 'invoke-method';
    name: string;
    parameters?: Array<InvokeMethodParameter>;
    then: Actions;
    error: Actions;
    condition?: string;
    schema: InvokeMethodSchema;
}
export interface InvokeMethodParameter {
    type: string;
    value: string;
    name: string;
}
export interface Execute {
    type: 'execute';
    code: string;
    condition?: string;
    then: Actions;
    error: Actions;
}
export interface Invoke {
    type: 'invoke';
    name: string;
    parameters?: Array<Parameter>;
    then: Actions;
    error: Actions;
    condition?: string;
}
export interface Export {
    type: 'export';
    name: string;
    format: string;
    parameters?: Array<Parameter>;
    then: Actions;
    error: Actions;
    condition?: string;
}
export interface Notify {
    type: 'notify';
    severity: string;
    summary: string;
    detail: string;
    duration?: number;
    condition?: string;
    then: Actions;
}
export interface Parameter {
    name?: string;
    value: string;
}
export interface Set {
    type: 'set';
    name: string;
    value: string;
    valueType?: string;
    condition?: string;
}
export interface Navigate {
    type: 'navigate';
    path: string;
    parameters?: Array<Parameter>;
    condition?: string;
}
export interface Back {
    type: 'back';
    condition?: string;
}
export interface Popup {
    type: 'popup';
    path: string;
    parameters?: Array<Parameter>;
    width?: number;
    height?: number;
    top: any;
    left: any;
    condition?: string;
    title?: string;
    then: Actions;
}
export declare function isPopup(action: Action): action is Popup;
export declare function isNavigate(action: Action): action is Navigate;
export declare function isNavigateBack(action: Action): action is Back;
export declare function isInvoke(action: Action): action is Invoke;
export declare function isExport(action: Action): action is Invoke;
export declare function isSet(action: Action): action is Set;
export declare function isNotify(action: Action): action is Notify;
export declare function isExecute(action: Action): action is Execute;
export declare function isClosePopup(action: Action): action is ClosePopup;
export declare function isInvokeMethod(action: Action): action is InvokeMethod;
