"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var get = require('lodash').get;
var _a = require('xml2js'), Parser = _a.Parser, Builder = _a.Builder;
var _b = require('semver'), gt = _b.gt, valid = _b.valid;
function parseXML(xml) {
    return new Promise(function (resolve, reject) {
        var parser = new Parser();
        parser.parseString(xml, function (err, result) {
            if (err) {
                reject(err);
            }
            else {
                resolve(result);
            }
        });
    });
}
function getPackageReferences(dom) {
    var itemGroups = get(dom, 'Project.ItemGroup');
    if (!itemGroups) {
        return [];
    }
    var packageReferenceItemGroup = itemGroups.find(function (itemGroup) {
        return itemGroup.hasOwnProperty('PackageReference');
    });
    if (!packageReferenceItemGroup) {
        return [];
    }
    return packageReferenceItemGroup.PackageReference;
}
function diff(sourcePackages, targetPackages) {
    var missingFromTarget = [];
    var needUpgrade = [];
    sourcePackages.forEach(function (sourcePackage) {
        var name = packageName(sourcePackage);
        var version = packageVersion(sourcePackage);
        var targetPackage = targetPackages.find(function (candidate) {
            return packageName(candidate) == name;
        });
        if (targetPackage) {
            var targetVersion = packageVersion(targetPackage);
            if (valid(targetVersion) && gt(version, targetVersion)) {
                needUpgrade.push(sourcePackage);
            }
        }
        else {
            missingFromTarget.push(sourcePackage);
        }
    });
    return {
        missingFromTarget: missingFromTarget,
        needUpgrade: needUpgrade
    };
}
function packageName(packageInfo) {
    return get(packageInfo, '$.Include');
}
function packageVersion(packageInfo) {
    return get(packageInfo, '$.Version', '').replace('-*', '');
}
function mergeProject(source, target) {
    return __awaiter(this, void 0, void 0, function () {
        var sourceDOM, targetDOM, sourcePackages, targetPackages, diffResult, builder;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    if (!target) {
                        return [2 /*return*/, source];
                    }
                    return [4 /*yield*/, parseXML(source)];
                case 1:
                    sourceDOM = _a.sent();
                    return [4 /*yield*/, parseXML(target)];
                case 2:
                    targetDOM = _a.sent();
                    sourcePackages = getPackageReferences(sourceDOM);
                    targetPackages = getPackageReferences(targetDOM);
                    diffResult = diff(sourcePackages, targetPackages);
                    targetPackages.splice.apply(targetPackages, [0, 0].concat(diffResult.missingFromTarget));
                    diffResult.needUpgrade.forEach(function (sourcePackage) {
                        var name = packageName(sourcePackage);
                        var index = targetPackages.findIndex(function (candidate) { return packageName(candidate) == name; });
                        targetPackages.splice(index, 1, sourcePackage);
                    });
                    builder = new Builder({
                        headless: true
                    });
                    return [2 /*return*/, builder
                            .buildObject(targetDOM)
                            .replace(/\/>/g, ' />')
                            .replace(/netcoreapp3.0/, 'netcoreapp3.1')];
            }
        });
    });
}
exports.mergeProject = mergeProject;
function mergeResx(source, target) {
    return __awaiter(this, void 0, void 0, function () {
        var sourceDOM, targetDOM, sourceData, targetData, missingFromTarget, builder;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    if (!target) {
                        return [2 /*return*/, source];
                    }
                    return [4 /*yield*/, parseXML(source)];
                case 1:
                    sourceDOM = _a.sent();
                    return [4 /*yield*/, parseXML(target)];
                case 2:
                    targetDOM = _a.sent();
                    sourceData = sourceDOM.root.data || [];
                    targetData = targetDOM.root.data || [];
                    missingFromTarget = [];
                    sourceData.forEach(function (sd) {
                        if (!targetData.find(function (td) { return td.$.name == sd.$.name; })) {
                            missingFromTarget.push(sd);
                        }
                    });
                    targetData.splice.apply(targetData, [0, 0].concat(missingFromTarget));
                    builder = new Builder({
                        headless: true
                    });
                    return [2 /*return*/, builder.buildObject(targetDOM).replace(/\/>/g, ' />')];
            }
        });
    });
}
exports.mergeResx = mergeResx;
//# sourceMappingURL=merge-project.js.map